@@ -174,7 +174,7 @@ function EnzymeInterpreter(
 
     return EnzymeInterpreter(
         cache_or_token,
-	mt == nothing ? Core.Compiler.InternalMethodTable(world) : Core.Compiler.OverlayMethodTable(world, mt),
+    mt == nothing ? Core.Compiler.InternalMethodTable(world) : Core.Compiler.OverlayMethodTable(world, mt),
 
         # Initially empty cache
         Vector{InferenceResult}(),
@@ -800,8 +800,8 @@ end
 # Override Base.copyto!(dest::AbstractArray, bc::Broadcasted{Nothing}) with
 #  a form which provides better analysis of loop indices
 @inline function override_bc_copyto!(dest::AbstractArray, bc::Base.Broadcast.Broadcasted{Nothing})
-	axdest = Base.axes(dest)
-	axbc = Base.axes(bc)
+    axdest = Base.axes(dest)
+    axbc = Base.axes(bc)
     axdest == axbc || Base.Broadcast.throwdm(axdest, axbc)
 
     if bc.args isa Tuple{AbstractArray}
@@ -826,20 +826,20 @@ end
     prev = nothing
     todo = Tuple{Expr, Type}[]
     for i in 1:length(x.parameters)
-	push!(todo, (:(x[$i]), x.parameters[i]))
+    push!(todo, (:(x[$i]), x.parameters[i]))
     end
     while length(todo) != 0
-	expr, ty = pop!(todo)
+    expr, ty = pop!(todo)
         if ty <: Number || ty <: Base.RefValue
             continue
         end
-	if ty <: Base.Broadcast.Broadcasted{<:Base.Broadcast.DefaultArrayStyle, Nothing}
-	    for i in 1:length(ty.parameters[4].parameters)
-	       push!(todo, (:($expr.args[$i]), ty.parameters[4].parameters[i]))
-	    end
-	    continue
-	end
-	@assert ty <: AbstractArray
+    if ty <: Base.Broadcast.Broadcasted{<:Base.Broadcast.DefaultArrayStyle, Nothing}
+        for i in 1:length(ty.parameters[4].parameters)
+           push!(todo, (:($expr.args[$i]), ty.parameters[4].parameters[i]))
+        end
+        continue
+    end
+    @assert ty <: AbstractArray
         if prev == nothing
             prev = quote
                 sz = size($expr)
@@ -857,10 +857,10 @@ end
         end
     end
     if result == :true
-	return quote
-	   Base.@_inline_meta
-   	   true
-  	end
+    return quote
+       Base.@_inline_meta
+       true
+    end
     end
     return quote
         Base.@_inline_meta
@@ -874,28 +874,28 @@ end
     prev = nothing
     todo = Tuple{Expr, Type}[]
     for i in 1:length(x.parameters)
-	push!(todo, (:(x[$i]), x.parameters[i]))
+    push!(todo, (:(x[$i]), x.parameters[i]))
     end
     while length(todo) != 0
-	expr, ty = pop!(todo)
+    expr, ty = pop!(todo)
         if ty <: Number || ty <: Base.RefValue
             continue
         end
-	if ty <: Base.Broadcast.Broadcasted{<:Base.Broadcast.DefaultArrayStyle, Nothing}
-	    for i in 1:length(ty.parameters[4].parameters)
-	       push!(todo, (:($expr.args[$i]), ty.parameters[4].parameters[i]))
-	    end
-	    continue
-	end
-	@assert ty <: AbstractArray
-	return quote
-	    Base.@_inline_meta
-	    $expr
-	end
+    if ty <: Base.Broadcast.Broadcasted{<:Base.Broadcast.DefaultArrayStyle, Nothing}
+        for i in 1:length(ty.parameters[4].parameters)
+           push!(todo, (:($expr.args[$i]), ty.parameters[4].parameters[i]))
+        end
+        continue
+    end
+    @assert ty <: AbstractArray
+    return quote
+        Base.@_inline_meta
+        $expr
+    end
     end
     return quote
         Base.@_inline_meta
-	throw(AssertionError("No array"))
+    throw(AssertionError("No array"))
     end
 end
 
@@ -933,12 +933,12 @@ end
     dest = @inline similar(bc, ElType)
     if same_sized(bc.args)
         # dest = @inline similar(first_array(bc.args), ElType)
-	@inbounds for I in 1:length(bc)
-	    val = overload_broadcast_getindex(bc, I)
+    @inbounds for I in 1:length(bc)
+        val = overload_broadcast_getindex(bc, I)
             dest[I] = val
-	    @simdloop
+        @simdloop
         end
-	return dest
+    return dest
     else
        # The existing code is rather slow for broadcast in practice: https://github.com/EnzymeAD/Enzyme.jl/issues/1434
        src = @inline Base.Broadcast.preprocess(nothing, bc)
@@ -956,17 +956,17 @@ end
     v = op(init, y[1])
    
     if same_sized(itr.args)
-	@inbounds for I in 2:length(itr)
-	    val = overload_broadcast_getindex(itr, I)
+    @inbounds for I in 2:length(itr)
+        val = overload_broadcast_getindex(itr, I)
             v = op(v, val)
-	    @simdloop
+        @simdloop
         end
     else
-	while true
-	    y = iterate(itr, y[2])
-	    y === nothing && break
-	    v = op(v, y[1])
-	end
+    while true
+        y = iterate(itr, y[2])
+        y === nothing && break
+        v = op(v, y[1])
+    end
     end
     return v
 end
@@ -983,7 +983,7 @@ end
         for i = ifirst + 1 : ilast
             @inbounds ai = A[i]
             v = op(v, f(ai))
-	    @simdloop
+        @simdloop
         end
         return v 
     end
@@ -1004,7 +1004,7 @@ end
         while i < last(inds) 
             @inbounds Ai = A[i+=1]
             s = op(s, f(Ai))
-	    @simdloop
+        @simdloop
         end
         return s
     end
@@ -1042,7 +1042,7 @@ end
        (!midnothing && Ty <: Base.Broadcast.Broadcasted{<:Base.Broadcast.DefaultArrayStyle})
         return all(Base.Fix2(bc_or_array_or_number_ty, midnothing), Ty.parameters[4].parameters)
     else
-	return Ty <: AbstractArray || Ty <: Number || Ty <: Base.RefValue
+    return Ty <: AbstractArray || Ty <: Number || Ty <: Base.RefValue
     end
 end
 
@@ -1051,7 +1051,7 @@ end
        (!midnothing && Ty <: Base.Broadcast.Broadcasted{<:Base.Broadcast.DefaultArrayStyle})
         return any(Base.Fix2(has_array, midnothing), Ty.parameters[4].parameters)
     else
-	return Ty <: AbstractArray
+    return Ty <: AbstractArray
     end
 end
 
@@ -1071,23 +1071,146 @@ end
     end
 end
 
-
+if VERSION < v"1.12"
 ## Computation of inferred result type, for empty and concretely inferred cases only
-ty_broadcast_getindex_eltype(interp, bc::Type{<:Base.Broadcast.Broadcasted}) = ty_combine_eltypes(interp, bc.parameters[3], (bc.parameters[4].parameters...,))
-ty_broadcast_getindex_eltype(interp, A) = eltype(A)  # Tuple, Array, etc.
+ty_broadcast_getindex_eltype(state::NamedTuple, bc::Type{<:Base.Broadcast.Broadcasted}) = ty_combine_eltypes(state, bc.parameters[3], (bc.parameters[4].parameters...,))
+ty_broadcast_getindex_eltype(state::NamedTuple, A) = eltype(A)  # Tuple, Array, etc.
 
-ty_eltypes(interp, ::Tuple{}) = Tuple{}
-ty_eltypes(interp, t::Tuple{Any}) = Iterators.TupleOrBottom(ty_broadcast_getindex_eltype(interp, t[1]))
-ty_eltypes(interp, t::Tuple{Any,Any}) = Iterators.TupleOrBottom(ty_broadcast_getindex_eltype(interp, t[1]), ty_broadcast_getindex_eltype(interp, t[2]))
-ty_eltypes(interp, t::Tuple) = (TT = ty_eltypes(interp, Base.tail(t)); TT === Union{} ? Union{} : Iterators.TupleOrBottom(ty_broadcast_getindex_eltype(interp, t[1]), TT.parameters...))
-# eltypes(t::Tuple) = Iterators.TupleOrBottom(ntuple(i -> _broadcast_getindex_eltype(t[i]), Val(length(t)))...)
+ty_eltypes(state::NamedTuple, ::Tuple{}) = Tuple{}
+ty_eltypes(state::NamedTuple, t::Tuple{Any}) = Iterators.TupleOrBottom(ty_broadcast_getindex_eltype(state, t[1]))
+ty_eltypes(state::NamedTuple, t::Tuple{Any,Any}) = Iterators.TupleOrBottom(ty_broadcast_getindex_eltype(state, t[1]), ty_broadcast_getindex_eltype(state, t[2]))
+ty_eltypes(state::NamedTuple, t::Tuple) = (TT = ty_eltypes(state, Base.tail(t)); TT === Union{} ? Union{} : Iterators.TupleOrBottom(ty_broadcast_getindex_eltype(state, t[1]), TT.parameters...))
 
 # Inferred eltype of result of broadcast(f, args...)
-function ty_combine_eltypes(interp, f, args::Tuple)
-    argT = ty_eltypes(interp, args)
+function ty_combine_eltypes(state::NamedTuple, f, args::Tuple)
+    (; interp, sv, max_methods) = state
+    argT = ty_eltypes(state, args)
     argT === Union{} && return Union{}
-    preprom = Core.Compiler._return_type(interp, Tuple{f, argT.parameters...})
-    return Base.promote_typejoin_union(preprom)
+    preprom = abstract_call(
+        interp,
+        ArgInfo(nothing, Any[f, argT.parameters...]),
+        StmtInfo(true),
+        sv,
+        max_methods,
+    )
+    return Base.promote_typejoin_union(widenconst(preprom.rt))
+end
+else
+## Computation of inferred result type, for empty and concretely inferred cases only
+ty_broadcast_getindex_eltype(state::NamedTuple, bc::Type{<:Base.Broadcast.Broadcasted})::Core.Compiler.Future{Type} = ty_combine_eltypes(state, bc.parameters[3], (bc.parameters[4].parameters...,))
+ty_broadcast_getindex_eltype(state::NamedTuple, A)::Core.Compiler.Future{Type} = Core.Compiler.Future{Type}(eltype(A))  # Tuple, Array, etc.
+
+ty_eltypes(state::NamedTuple, ::Tuple{})::Core.Compiler.Future{Type} = Tuple{}
+function ty_eltypes(state::NamedTuple, t::Tuple{Any})::Core.Compiler.Future{Type}
+    retT = ty_broadcast_getindex_eltype(state, t[1])
+    (; interp, sv) = state
+    Core.Compiler.Future{Type}(retT, interp, sv) do retT, interp, sv
+        Iterators.TupleOrBottom(retT)
+    end
+end
+function ty_eltypes(state::NamedTuple, t::Tuple{Any,Any})::Core.Compiler.Future{Type}
+    retT1 = ty_broadcast_getindex_eltype(state, t[1])
+    retT2 = ty_broadcast_getindex_eltype(state, t[2])
+    (; interp, sv) = state
+    Core.Compiler.Future{Type}(isready(retT1) && isready(retT2), interp, sv) do interp, sv
+            Iterators.TupleOrBottom(retT1[], retT2[])
+    end
+end
+function ty_eltypes(state::NamedTuple, t::Tuple)::Core.Compiler.Future{Type}
+    TT = ty_eltypes(state, Base.tail(t))
+    (; interp, sv) = state
+    if TT === Union{}
+        Core.Compiler.Future{Type}{Union{}}
+    else
+        retT = ty_broadcast_getindex_eltype(state, t[1])
+        Core.Compiler.Future{Type}(isready(TT) && isready(retT), interp, sv) do interp, sv
+            Iterators.TupleOrBottom(retT[], TT[].parameters...)
+        end
+    end
+end
+
+# Inferred eltype of result of broadcast(f, args...)
+function ty_combine_eltypes(state::NamedTuple, f, args::Tuple)::Core.Compiler.Future{Type}
+    (; interp, sv, max_methods) = state
+    argT = ty_eltypes(state, args)
+    ret = Core.Compiler.Future{Type}()
+    function finish_abstract_call(interp, sv)
+        isready(argT) || return false
+        if argT[] == Union{}
+            ret[] = Union{}
+            return true
+        end
+        preprom = abstract_call(
+            interp,
+            ArgInfo(nothing, Any[f, argT[].parameters...]),
+            StmtInfo(true, false),
+            sv,
+            max_methods,
+        )
+        function continuation(interp, sv)
+            isready(preprom) || return false
+            retty = widenconst(preprom[].rt)
+            ret[] = Base.promote_typejoin_union(retty)
+            return true
+        end
+        continuation(interp, sv) || push!(sv.tasks, continuation)
+        return true
+    end
+    finish_abstract_call(interp, sv) || push!(sv.tasks, finish_abstract_call)
+    return ret
+end
+
+struct broadcast_rewriter
+    ret::Core.Compiler.Future{CallMeta}
+    ElType::Core.Compiler.Future{Type}
+    fargs::Vector{Any}
+    argtypes
+    si::StmtInfo
+    max_methods::Int
+    f::Any
+    arginfo::ArgInfo
+end
+function (bcr::broadcast_rewriter)(interp, sv)
+    (; ret, ElType, fargs, argtypes, si, max_methods, f, arginfo) = bcr
+    isready(ElType) || return false
+    ElType = ElType[]
+    local retFuture
+    if ElType !== Union{} && Base.isconcretetype(ElType)
+        fn2 = Enzyme.Compiler.Interpreter.OverrideBCMaterialize{ElType}()
+        arginfo2 = ArgInfo(
+            fargs isa Nothing ? nothing : [:(fn2), fargs[2:end]...],
+           [Core.Const(fn2), argtypes[2:end]...],
+        )
+        retFuture = Base.@invoke abstract_call_known(
+            interp::AbstractInterpreter,
+            fn2::Any,
+            arginfo2::ArgInfo,
+            si::StmtInfo,
+            sv::AbsIntState,
+            max_methods::Int,
+        )
+    else 
+        if interp.handler != nothing
+            return interp.handler(interp, f, arginfo, si, sv, max_methods)
+        end
+        retFuture = Base.@invoke abstract_call_known(
+            interp::AbstractInterpreter,
+            f::Any,
+            arginfo::ArgInfo,
+            si::StmtInfo,
+            sv::AbsIntState,
+            max_methods::Int,
+        )
+    end
+    
+    function continuation(interp, sv)
+        isready(retFuture) || return false
+        ret[] = retFuture[]
+        return true
+    end
+    continuation(interp, sv) || push!(sv.tasks, continuation)
+    return true
+end
 end
 
 function abstract_call_known(
@@ -1135,29 +1258,6 @@ function abstract_call_known(
     end
     
     if interp.broadcast_rewrite
-        if f === Base.materialize && length(argtypes) == 2
-            bcty = widenconst(argtypes[2])
-    	    if Base.isconcretetype(bcty) && bcty <: Base.Broadcast.Broadcasted{<:Base.Broadcast.DefaultArrayStyle, Nothing} && bc_or_array_or_number_ty(bcty) && has_array(bcty)
-        		ElType = ty_broadcast_getindex_eltype(interp, bcty)
-        		if ElType !== Union{} && Base.isconcretetype(ElType)
-        		    fn2 = Enzyme.Compiler.Interpreter.OverrideBCMaterialize{ElType}()
-                    arginfo2 = ArgInfo(
-                        fargs isa Nothing ? nothing : [:(fn2), fargs[2:end]...],
-        	           [Core.Const(fn2), argtypes[2:end]...],
-                    )
-
-                    return Base.@invoke abstract_call_known(
-                        interp::AbstractInterpreter,
-                        fn2::Any,
-                        arginfo2::ArgInfo,
-                        si::StmtInfo,
-                        sv::AbsIntState,
-                        max_methods::Int,
-                    )
-                end
-            end
-        end
-
         if f === Base.copyto! && length(argtypes) == 3
             # Ideally we just override uses of the AbstractArray base class, but
             # I don't know how to override the method in base, without accidentally overridding
@@ -1181,12 +1281,12 @@ function abstract_call_known(
                 )
             end
         end
-	
-	if f === Base.mapreduce_impl &&  length(argtypes) == 7
+    
+    if f === Base.mapreduce_impl &&  length(argtypes) == 7
             if widenconst(argtypes[4]) <: Array && 
-	       widenconst(argtypes[5]) <: Integer &&
-	       widenconst(argtypes[6]) <: Integer &&
-	       widenconst(argtypes[7]) <: Int 
+           widenconst(argtypes[5]) <: Integer &&
+           widenconst(argtypes[6]) <: Integer &&
+           widenconst(argtypes[7]) <: Int 
                 arginfo2 = ArgInfo(
                     fargs isa Nothing ? nothing :
                     [:(Enzyme.Compiler.Interpreter.override_bc_mapreduceimpl), fargs[2:end]...],
@@ -1203,10 +1303,10 @@ function abstract_call_known(
                 )
             end
         end
-	
-	if f === Base._mapreduce &&  length(argtypes) == 5
+    
+    if f === Base._mapreduce &&  length(argtypes) == 5
             if widenconst(argtypes[4]) <: Base.IndexLinear &&
-	       widenconst(argtypes[5]) <: Array
+           widenconst(argtypes[5]) <: Array
                 arginfo2 = ArgInfo(
                     fargs isa Nothing ? nothing :
                     [:(Enzyme.Compiler.Interpreter.override_bc_mapreduce), fargs[2:end]...],
@@ -1224,14 +1324,14 @@ function abstract_call_known(
             end
         end
        
-	if f === Base._foldl_impl &&  length(argtypes) == 4
-	    
-	    bcty = widenconst(argtypes[4])
+    if f === Base._foldl_impl &&  length(argtypes) == 4
+        
+        bcty = widenconst(argtypes[4])
 
 
             if widenconst(argtypes[3]) <: Base._InitialValue &&
-	       bcty <: Base.Broadcast.Broadcasted{<:Base.Broadcast.DefaultArrayStyle} && ndims(bcty) >= 2 &&
-	       bc_or_array_or_number_ty(bcty, false) && has_array(bcty, false)
+           bcty <: Base.Broadcast.Broadcasted{<:Base.Broadcast.DefaultArrayStyle} && ndims(bcty) >= 2 &&
+           bc_or_array_or_number_ty(bcty, false) && has_array(bcty, false)
            
                 arginfo2 = ArgInfo(
                     fargs isa Nothing ? nothing :
@@ -1273,10 +1373,47 @@ function abstract_call_known(
             )
         end
     end
+    if interp.broadcast_rewrite && f === Base.materialize && length(argtypes) == 2
+            bcty = widenconst(argtypes[2])
+            if Base.isconcretetype(bcty) && bcty <: Base.Broadcast.Broadcasted{<:Base.Broadcast.DefaultArrayStyle, Nothing} && bc_or_array_or_number_ty(bcty) && has_array(bcty)
+                ElType = ty_broadcast_getindex_eltype((; interp, sv, max_methods), bcty)
+                if VERSION < v"1.12"
+                    if ElType !== Union{} && Base.isconcretetype(ElType)
+                        fn2 = Enzyme.Compiler.Interpreter.OverrideBCMaterialize{ElType}()
+                        arginfo2 = ArgInfo(
+                            fargs isa Nothing ? nothing : [:(fn2), fargs[2:end]...],
+                           [Core.Const(fn2), argtypes[2:end]...],
+                        )
 
+                        return Base.@invoke abstract_call_known(
+                            interp::AbstractInterpreter,
+                            fn2::Any,
+                            arginfo2::ArgInfo,
+                            si::StmtInfo,
+                            sv::AbsIntState,
+                            max_methods::Int,
+                        )
+                    end
+                else
+                    ret = Core.Compiler.Future{Core.Compiler.CallMeta}()
+                    bcr = broadcast_rewriter(
+                                ret,
+                                ElType,
+                                fargs,
+                                argtypes,
+                                si,
+                                max_methods,
+                                f,
+                                arginfo)
+                    bcr = bcr(interp, sv) || push!(sv.tasks, bcr)
+                    return ret
+                end
+            end
+        end
     if interp.handler != nothing
         return interp.handler(interp, f, arginfo, si, sv, max_methods)
     end
+    
     return Base.@invoke abstract_call_known(
         interp::AbstractInterpreter,
         f::Any,
